% Example 10.14

% Exploratory Data Analysis with MATLAB, 2nd Edition
% Wendy L. and Angel R. Martinez and Jeff Solka

% This example demonstrates the Andrews Image

load iris
data = [setosa;versicolor;virginica];

% Get a vector of class labels. These will be indices to the colormap.
clabs = [56*ones(50,1); 40*ones(50,1); 8*ones(50,1)];

% Let's re-order the rows of the data
% matrix to make it more interesting.
rp = randperm(150);
data = data(rp,:);

% Also re-order the class labels.
clabs = clabs(rp);


% Now get the values of the curves as 
% rows of a matrix A.
[A,t] = andrewsvals(data);

% Create the Andrews' image.
imagesc(A)
xlabel('t')
ylabel('Observation Number')


% Cluster the raw data according to agglomerative clustering. 
% Then order the data by the dendrogram leaves. 
ydist = pdist(data);
Z = linkage(ydist,'complete');
figure
subplot(1,2,1)
[H, T, perm] = dendrogram(Z,0,'orientation','left');
axis off
subplot(1,2,2)
him = imagesc(flipud(A(perm,:)));

title('Andrews'' Image')
xstr = [''; ''; '']; 
set(gca,'xticklabel',xstr)


% This adds a colorbar where the colors indicate the
% class label.

% positions for axes.
posim =   [ 0.5703    0.1100    0.3    0.8150];
poscb =   [ 0.9    0.1100    0.04    0.8150];
set(get(him,'parent'),'position',posim);
% Get axes for the colorbar
hax = axes('position',poscb,'box','on');
% set(hax,'position',poscb);
% Now, do the Andrews' image stuff, but reorder rows of clabs accordingly
% and plot. 
hcb = image(clabs(fliplr(perm)));
axis off
title('Class IDs')



