% Example 5.4

% Exploratory Data Analysis with MATLAB, 2nd Edition
% Wendy L. and Angel R. Martinez and Jeff Solka

% Illustrates the application of nonnegative matrix factorization
% to document clustering.

% First, we load the dataset that has the term-document
% matrix.
load nmfclustex
% Next, normalize each column to have unit norm.
[n,p] = size(nmfclustex);
for i = 1:p
	termdoc(:,i) = nmfclustex(:,i)/...
        (norm(nmfclustex(:,i)));
end
% Here we perform nonnegative matrix factorization with
% k = 2 to elucidate the cluster structure. Notice that
% nmf is formulated as X = U * V, so we use VT to stand
% for V to make the notation a little easier
[U,VT] = nnmf(termdoc,2,'algorithm','mult');
% It is easier to proceed forward with our calculations if we 
% have V
V = VT';
[nu,pu] = size(U);
[nv,pv] = size(V);

%Normalize the V entries as discussed above
for i = 1:nv
	for j = 1:pv
		V(i,j) = V(i,j) * norm(U(:,j));
	end 
end
%Normalize the U entries as discussed above
for j = 1:pu
		U(:,j) = U(:,j)/(norm(U(:,j)));
end

% Here we plot to illustrate that the document cluster 
% structure is easily discernable in the NMF space.
% The first 5 documents are plotted as plus signs
% and the last 4 documents are plotted as circles
labs = {'d1','d2','d3','d4','d5','d6','d7','d8','d9'};
plot(V(1:5,1), V(1:5,2),'r*')
text(V(1:5,1)+.05, V(1:5,2),labs(1:5))
hold on
plot(V(6:9,1), V(6:9,2),'bo')
text(V(6:9,1), V(6:9,2)+.05,labs(6:9))
xlabel('V1')
ylabel('V2')
hold off

% Here we extract the axes corresponding to each document.
[Y,I] = max(V,[],2);
I'

% We notice once again that the first 5 docs cluster on V2,
% while the last 4 docs cluster on V1, the x-axis


